@@ -25,7 +25,7 @@ except:
 from pydap.client import open_url
 
 try:
-    from ringslegacy import okuboweiss
+    from rings import okuboweiss
 except:
     pass
 
@@ -71,6 +71,11 @@ class AVISO_fetch(object):
         if 'force_download' not in self.cfg:
             self.cfg['force_download'] = False
 
+        if (self.cfg['datadir'] != '') and \
+                (not os.path.isdir(self.cfg['datadir'])):
+            print "There is no data directory: %s" % self.cfg['datadir']
+            return
+
         self.file = os.path.join(self.cfg['datadir'], self.cfg['filename'])
         try:
             self.nc = netCDF4.Dataset(self.file,'w', format='NETCDF4')
@@ -85,11 +90,6 @@ class AVISO_fetch(object):
         self.nc.metadata_force_download = str(self.cfg['force_download'])
         # ----------
 
-        self.set_source_filename()
-        self.set_dataset()
-
-        self.download_time()
-        self.download_LonLat()
         self.download_data()
 
         self.nc.close()
@@ -139,28 +139,35 @@ class AVISO_fetch(object):
         self.logger = logger
 
 
-    def set_source_filename(self):
+    def set_source_filename(self, map):
         """
         """
-        self.cfg['source_filename'] = "dataset-duacs-dt-%s-global-merged-%s" % (self.cfg['type'], self.cfg['map'])
+        assert (map in ['madt', 'msla'])
+
+        self.cfg['source_filename'] = "dataset-duacs-dt-%s-global-merged-%s" % (self.cfg['type'], map)
         self.logger.debug("source_filename: %s" % self.cfg['source_filename'])
 
-    def set_dataset(self):
+    def set_dataset(self, map, var):
         """
         """
+        assert (var in ['h', 'u', 'v'])
         self.logger.debug("Setting the dataset on the DAP server")
+        self.set_source_filename(map)
 
-        self.url = {'h': "%s/%s-h-daily" % (self.cfg['urlbase'], self.cfg['source_filename']), 
-                'uv': "%s/%s-uv-daily" % (self.cfg['urlbase'], self.cfg['source_filename'])}
-        self.dataset = {'h': open_url(self.url['h']), 'uv': open_url(self.url['uv'])}
+        if var == 'h':
+            url = "%s/%s-h-daily" % (self.cfg['urlbase'], self.cfg['source_filename'])
+        if var in ['u', 'v']:
+            url = "%s/%s-uv-daily" % (self.cfg['urlbase'], self.cfg['source_filename'])
 
+        dataset = open_url(url)
         self.logger.debug("Dataset on the DAP server is ready")
+        return dataset
 
-    def download_time(self):
+    def download_time(self, dataset):
         """
         """
         self.logger.debug("Downloading time")
-        t = self.dataset['h']['time'][:]
+        t = dataset['time'][:]
         # Improve it and extract this date from the DAP server
         d0=datetime(1950,1,1)
         if 't_ini' not in self.cfg['limits']:
@@ -180,7 +187,7 @@ class AVISO_fetch(object):
                 d = datetime.strptime(self.cfg['limits']['d_fin'], '%Y-%m-%d')
                 self.cfg['limits']['t_fin'] = np.nonzero(t>((d-d0).days*24))[0][0]
             else:
-                self.cfg['limits']['t_fin'] = self.dataset['h']['time'].shape[0]
+                self.cfg['limits']['t_fin'] = dataset['time'].shape[0]
                 self.logger.debug("Setting t_fin: %s" % self.cfg['limits']['t_fin'])
 
         t_ini = self.cfg['limits']['t_ini']
@@ -198,40 +205,40 @@ class AVISO_fetch(object):
         #else:
         #    self.logger.error("Problems interpreting the time")
 
-        t = self.dataset['h']['time'][t_ini:t_fin:t_step].tolist()
+        t = dataset['time'][t_ini:t_fin:t_step].tolist()
 
         self.nc.createDimension('time', len(range(t_ini,t_fin,t_step)))
-        nct = self.nc.createVariable('time', 'f', ('time', ))
+        nct = self.nc.createVariable('time', 'f8', ('time', ))
         nct[:] = t
-        nct.units = self.dataset['h']['time'].attributes['units']
+        nct.units = dataset['time'].attributes['units']
 
-    def download_LonLat(self):
+    def download_LonLat(self, dataset):
         """ Download the Lon x Lat coordinates
         """
         self.logger.debug("Downloading LonLat")
         data = {}
         limits = self.cfg['limits']
-        Lat = self.dataset['h']['NbLatitudes'][:].astype('f')
-        Lon = self.dataset['h']['NbLongitudes'][:].astype('f')
+        Lat = dataset['NbLatitudes'][:].astype('f')
+        Lon = dataset['NbLongitudes'][:].astype('f')
 
         # If data is requested as -180/180, convert to 0/360,
         #   which is the pattern in AVISO
-        if limits['lonini'] < 0: limits['lonini']+=360
-        if limits['lonfin'] < 0: limits['lonfin']+=360
+        if limits['LonIni'] < 0: limits['LonIni']+=360
+        if limits['LonFin'] < 0: limits['LonFin']+=360
 
-        Latlimits = numpy.arange(Lat.shape[0])[(Lat[:]>=limits["latini"]) & (Lat[:]<=limits["latfin"])]
+        Latlimits = numpy.arange(Lat.shape[0])[(Lat[:]>=limits["LatIni"]) & (Lat[:]<=limits["LatFin"])]
         Latlimits = [Latlimits[0],Latlimits[-1]]
 
         lat = Lat[Latlimits[0]:Latlimits[-1]+1]
 
-        if limits['lonfin'] > limits['lonini']:
-            Lonlimits = numpy.arange(Lon.shape[0])[(Lon[:]>=limits["lonini"]) & (Lon[:]<=limits["lonfin"])]
+        if limits['LonFin'] > limits['LonIni']:
+            Lonlimits = numpy.arange(Lon.shape[0])[(Lon[:]>=limits["LonIni"]) & (Lon[:]<=limits["LonFin"])]
             Lonlimits=[Lonlimits[0],Lonlimits[-1]]
 
             lon = Lon[Lonlimits[0]:Lonlimits[-1]+1]
         else:
-            Lonlimits = [np.nonzero(Lon>=limits['lonini'])[0][0], 
-                    np.nonzero(Lon<=limits['lonfin'])[0][-1]] 
+            Lonlimits = [np.nonzero(Lon>=limits['LonIni'])[0][0], 
+                    np.nonzero(Lon<=limits['LonFin'])[0][-1]] 
 
             lon = np.append(Lon[Lonlimits[0]:],Lon[:Lonlimits[1]+1])
                     
@@ -248,42 +255,78 @@ class AVISO_fetch(object):
         self.nc.createDimension('lat', lat.shape[0])
         self.nc.createDimension('lon', lon.shape[0])
 
-        ncLat = self.nc.createVariable('Lat', 'f', ('lat', 'lon'))
-        ncLon = self.nc.createVariable('Lon', 'f', ('lat', 'lon'))
+        ncLat = self.nc.createVariable('Lat', 'f4', ('lat', 'lon'))
+        ncLon = self.nc.createVariable('Lon', 'f4', ('lat', 'lon'))
 
         ncLat[:] = Lat
         ncLon[:] = Lon
 
     def download_data(self):
-        """ Download h and uv in blocks
+        """ 
+
+             Not a cute way, but works for now.
         """
-        # Will download blocks of at most 5MB
-        #   i.e. 4e7 floats of 32bits.
-        dblocks = min(100, max(1, int(4e7/self.slice_size)))
+        if self.cfg['map'] == 'madt+msla':
+            dataset = self.set_dataset('madt', 'h')
 
-        ti = numpy.arange(self.cfg['limits']['t_ini'], 
-                self.cfg['limits']['t_fin'], 
-                self.cfg['limits']['t_step'])
+            self.download_time(dataset)
+            self.download_LonLat(dataset)
 
-        blocks = ti[::dblocks]
-        if ti[-1] not in blocks:
-            blocks = numpy.append(blocks, self.cfg['limits']['t_fin'])
+            self.download_var('ssh', dataset['Grid_0001']['Grid_0001'], dataset['Grid_0001'].attributes)
+            dataset = self.set_dataset('madt', 'u')
+            self.download_var('u', dataset['Grid_0001']['Grid_0001'], dataset['Grid_0001'].attributes)
+            self.download_var('v', dataset['Grid_0002']['Grid_0002'], dataset['Grid_0002'].attributes)
 
-        ntries = 40
-        #------
-        data = {}
-        for v, dataset, missing_value in zip(['h','u','v'], 
-                [self.dataset['h']['Grid_0001']['Grid_0001'], 
-                    self.dataset['uv']['Grid_0001']['Grid_0001'], 
-                    self.dataset['uv']['Grid_0002']['Grid_0002']], 
-                [self.dataset['h']['Grid_0001']._FillValue, 
-                    self.dataset['uv']['Grid_0001']._FillValue, 
-                    self.dataset['uv']['Grid_0002']._FillValue]):
+            dataset = self.set_dataset('msla', 'h')
+            self.download_var('eta', dataset['Grid_0001']['Grid_0001'], dataset['Grid_0001'].attributes)
+            dataset = self.set_dataset('madt', 'u')
+            self.download_var('u_anom', dataset['Grid_0001']['Grid_0001'], dataset['Grid_0001'].attributes)
+            self.download_var('v_anom', dataset['Grid_0002']['Grid_0002'], dataset['Grid_0002'].attributes)
+            return
 
+        dataset = self.set_dataset(self.cfg['map'], 'h')
+
+        self.download_time(dataset)
+        self.download_LonLat(dataset)
+
+        self.download_var('h', dataset['Grid_0001']['Grid_0001'], dataset['Grid_0001'].attributes)
+        dataset = self.set_dataset(self.cfg['map'], 'u')
+        self.download_var('u', dataset['Grid_0001']['Grid_0001'], dataset['Grid_0001'].attributes)
+        self.download_var('v', dataset['Grid_0002']['Grid_0002'], dataset['Grid_0002'].attributes)
+
+    def download_var(self, v, dataset, attr):
+            # Will download blocks of at most 5MB
+            #   i.e. 4e7 floats of 32bits.
+            dblocks = min(100, max(1, int(4e7/self.slice_size)))
+
+            ti = numpy.arange(self.cfg['limits']['t_ini'],
+                    self.cfg['limits']['t_fin'],
+                    self.cfg['limits']['t_step'])
+
+            blocks = ti[::dblocks]
+            if ti[-1] not in blocks:
+                blocks = numpy.append(blocks, self.cfg['limits']['t_fin'])
+
+            #------
+            ntries = 40
             print "Getting %s" % v
             #data['h'] = ma.masked_all((len(ti),Lonlimits[-1]-Lonlimits[0], Latlimits[-1]-Latlimits[0]), dtype=numpy.float64)
-            data[v] = self.nc.createVariable(v, 'f', ('time', 'lat', 'lon'))
-            data[v].missing_value = missing_value
+            data = self.nc.createVariable(v, 'f4', ('time', 'lat', 'lon'),
+                    fill_value=attr['_FillValue'])
+            missing_value = attr['_FillValue']
+            #data.missing_value = missing_value
+
+            units = attr['units']
+            if units == 'cm':
+                factor = 1e-2
+                units = 'm'
+            elif units == 'cm/s':
+                factor = 1e-2
+                units = 'm/s'
+            else:
+                factor = None
+
+            data.units = units
 
             # Work on these limits. Must have a better way to handle it
             Lonlimits = self.cfg['limits']['Lonlimits']
@@ -301,7 +344,12 @@ class AVISO_fetch(object):
                             tmp1 = dataset[b1:b2:self.cfg['limits']['t_step'], Lonlimits[0]:,Latlimits[0]:Latlimits[-1]+1]
                             tmp2 = dataset[b1:b2:self.cfg['limits']['t_step'], :Lonlimits[-1]+1,Latlimits[0]:Latlimits[-1]+1]
                             tmp = np.append(tmp1, tmp2, axis=1)
-                        data[v][ind] = tmp.swapaxes(1,2).astype('f')
+
+                        if factor is not None:
+                            ind_valid = tmp != missing_value
+                            tmp[ind_valid] = factor * tmp[ind_valid]
+
+                        data[ind] = tmp.swapaxes(1,2).astype('f')
                         break
                     except:
                         waitingtime = 30+i*20
@@ -326,9 +374,9 @@ class Products(object):
         except:
             return
 
-        W = self.nc.createVariable('W', 'f', ('time', 'lat', 'lon'))
+        W = self.nc.createVariable('W', 'f4', ('time', 'lat', 'lon'))
         W.missing_value = self.nc.variables['u'].missing_value
-        zeta = self.nc.createVariable('zeta', 'f', ('time', 'lat', 'lon'))
+        zeta = self.nc.createVariable('zeta', 'f4', ('time', 'lat', 'lon'))
         zeta.missing_value = self.nc.variables['u'].missing_value
 
         for tn in range(self.nc.variables['time'].size):
@@ -416,7 +464,7 @@ class Aviso_map(object):
         if 'map' not in self.metadata:
             self.metadata['map'] = "madt"       # madt, msla
         if 'limits' not in self.metadata:
-            self.metadata['limits'] = {'latini': 0, 'latfin': 15, 'lonini': 296, 'lonfin': 317}
+            self.metadata['limits'] = {'LatIni': 0, 'LatFin': 15, 'LonIni': 296, 'LonFin': 317}
         if 'datadir' not in self.metadata:
             self.metadata['datadir'] = "./" #"../data"
         if 'urlbase' not in self.metadata:
@@ -500,9 +548,9 @@ class Aviso_map(object):
         limits=self.metadata['limits']
         Lat=dataset_h['NbLatitudes']
         Lon=dataset_h['NbLongitudes']
-        Latlimits=numpy.arange(Lat.shape[0])[(Lat[:]>=limits["latini"]) & (Lat[:]<=limits["latfin"])]
+        Latlimits=numpy.arange(Lat.shape[0])[(Lat[:]>=limits["LatIni"]) & (Lat[:]<=limits["LatFin"])]
         Latlimits=[Latlimits[0],Latlimits[-1]]
-        Lonlimits=numpy.arange(Lon.shape[0])[(Lon[:]>=limits["lonini"]) & (Lon[:]<=limits["lonfin"])]
+        Lonlimits=numpy.arange(Lon.shape[0])[(Lon[:]>=limits["LonIni"]) & (Lon[:]<=limits["LonFin"])]
         Lonlimits=[Lonlimits[0],Lonlimits[-1]]
 
         data['Lon'], data['Lat'] = numpy.meshgrid( (Lon[Lonlimits[0]:Lonlimits[-1]]), (Lat[Latlimits[0]:Latlimits[-1]]) )
@@ -535,7 +583,7 @@ class Aviso_map(object):
 
             print "Getting %s" % v
             #data['h'] = ma.masked_all((len(ti),Lonlimits[-1]-Lonlimits[0], Latlimits[-1]-Latlimits[0]), dtype=numpy.float64)
-            self.data[v] = nc.createVariable(v, 'f', ('time', 'lat', 'lon'))
+            self.data[v] = nc.createVariable(v, 'f4', ('time', 'lat', 'lon'))
             self.data[v].missing_value = missing_value
             for b1, b2 in zip(blocks[:-1], blocks[1:]):
                 print "From %s to %s of %s" % (b1, b2, blocks[-1])
@@ -552,14 +600,28 @@ class Aviso_map(object):
             #data['h'] = 1e-2*data['h'].swapaxes(1,2)
 
 
-def set_z(self):
-    """
+def mask_shallow(ncfile, zfile, zlimit, vars = []):
+    """ Mask all variables in vars @ gridpoints shallower than mindepth
 
          Use http://opendap.ccst.inpe.br/Misc/etopo2/ETOPO2v2c_f4.nc
     """
+    nc = netCDF4.Dataset(ncfile, 'a')
+    Lat = nc.variables['Lat'][:]
+    Lon = nc.variables['Lon'][:]
+
     #Lon,Lat = numpy.meshgrid(self.data['lon'],self.data['lat'])
     from fluid.common.common import get_bathymery
-    self.data['z'] = get_bathymery(self.data['Lat'], self.data['Lon'], etopo_file="/Users/castelao/work/misc/ETOPO2v2c_f4.nc")
+    z = get_bathymery(Lat, Lon, etopo_file=zfile)
+    ind = z > zlimit
+    for v in vars:
+        if nc.variables[v].shape[1:] != ind.shape:
+            return
+    I, J = np.nonzero(ind)
+    for i, j in zip(I, J):
+        for v in vars:
+            nc.variables[v][:,i,j] = nc.variables[v].missing_value
+    nc.sync()
+    nc.close()
 
 def mask(self):
     """ Improve it. Make it more flexible
